package Stanford::FlexTest::Nagios ;

use strict;
use warnings;
use autodie;

## no critic (RequireArgUnpacking);

# Make this a Moose class
use Moose;
use namespace::autoclean;

use Carp ;
use IPC::Run qw/ run timeout / ;

has verbose => (
    is       => 'rw',
    isa      => 'Bool',
    required => 1,
    default  => 0,
);

has hostname => (
    is       => 'rw',
    isa      => 'Str',
    required => 1,
    default  => 'unknown',
);

has nagios_servers => (
    is       => 'rw',
    isa      => 'ArrayRef[Str]',
    required => 1,
    default  => sub { [] },
);

sub exit_with_error {
    my ($msg) = @_;
    print "error: $msg.\n";
    exit 1;
}

sub progress {
    my ($self) = shift ;

    my ($msg) = @_ ;
    if ($self->verbose) {
	print $msg . "\n" ;
    }
    return ;
}

sub process_flextest_results {
    my ($self) = shift ;

    my ($nagios_test_name, $flextests, $aterror) = @_ ;

    if ($aterror) {
        $self->send_unreachable_passive_to_nagios_server($nagios_test_name) ;
        my $msg = "error with '$nagios_test_name': $aterror" ;
        $self->exit_with_error($msg) ;
    } else {
        $self->send_nagios_result($flextests, $nagios_test_name) ;
    }

    return ;
}

sub send_nagios_result {
    my ($self) = shift ;

    my ($flextests, $nagios_test_name) = @_ ;

    # Get the last flextest in $flextests. If it is a failure, then send
    # a CRITICAL to the Nagios server.
    my $last_flextest  = $flextests->last_flextest() ;
    my $msg ;
    my $detail_message = $last_flextest->msg() ;
    if ($last_flextest->failed()) {
	$msg = "2;CRITICAL - '$detail_message' failed" ;
    } else {
	$msg = '0;OK' ;
    }

    $self->send_passive_to_nagios_server($nagios_test_name, $msg) ;
    return ;
}


sub send_passive_to_nagios_server {
    my ($self) = shift ;

    my ($nagios_test_name, $msg) = @_ ;

    if (!$nagios_test_name) {
        croak 'missing required Nagios test name' ;
    }

    my $hostname = $self->hostname ;

    my $psv_msg = "$hostname;$nagios_test_name;$msg" ;
    $self->progress("passive message: $psv_msg") ;

    my $out ;
    my @nagios_servers = @{ $self->nagios_servers } ;

    foreach my $nagios_server (@nagios_servers) {
        my @command = (
            '/usr/sbin/send_nsca',
            '-d', q{;},
            '-H', $nagios_server,
            ) ;
        IPC::Run::run \@command, \$psv_msg, \$out or croak $? ;
        $self->progress("send_ncsa command to $nagios_server output: $out") ;
    }

    return ;
}


sub send_unreachable_passive_to_nagios_server {
    my ($self) = shift ;

    my ($nagios_test_name) = @_ ;

    my $msg = '3;UNKNOWN - test failed to run properly' ;
    return $self->send_passive_to_nagios_server($nagios_test_name, $msg) ;
}


1 ;
